/*************************************************************************
  Copyright (c) 1993 - 1997
  Digital Equipment Corporation, Maynard, Mass.

  This software is furnished under a license and may be used and  copied
  only  in  accordance  with  the  terms  of  such  license and with the
  inclusion of the above copyright notice.  This software or  any  other
  copies  thereof may not be provided or otherwise made available to any
  other person.  No title to and ownership of  the  software  is  hereby
  transferred.

  The information in this software is subject to change  without  notice
  and  should  not  be  construed  as  a commitment by DIGITAL Equipment
  Corporation.
 *************************************************************************/


/*++
* ==============
* = xmt_pace.c =
* ==============
*
* Description:
*   This application demonstrates the API's ability to establish a UBR PVC
*   and pace that PVC.  A paced UBR PVC is a circuit whose cell rate never
*   exceeds the rate specified by the user despite what best effort might
*   produce.  A paced UBR circuit may be able to run continuously at the
*   user-specified rate.  This application is used in conjunction with the
*   api_rcv_pace executable compiled from rcv_pace.c.
*
* Authors:
*   PAE - Paul A. Ely
*
* Modification History:
*   Date          Name     Description
*   15-Jul-1997   PAE      Created.
*   23-Jul-1997   PAE      Modified application to allow user-specifiable
*                          packet sizes to 64K bytes.
*
* Arguments:
*   driver name   - The driver service name, e.g. lpa0 or lta0.
*   vpi           - The VPI to use in the PVC.
*   vci           - The VCI to use in the PVC.
*   pkt_size      - The packet size in bytes, 0<pkt_size<65536.
*   kbps          - The maximum kilobit/sec rate desired.
*
* Issues:
*   none.
--*/

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <sys/errno.h>
#include <netaal/aal.h>

#define errtext(n) n
#define SIZE         65535

int main (int argc, char *argv[])
   {
   int                  a, cell_rate,sock;
   int                  i, bit_rate;
   unsigned int         value, pkt_size;
   char                 buff[SIZE];
   char                 drv_name[32];
   char                 instance;
   ATM_CONNECTION_ID    conn_id;
   ATM_PARAMS           params;
   ATM_DEVICE_INFO      device_info;

   /* Clear the driver name array before use. */

   for (i = 0; i < 32; i++)
      drv_name[i] = '\0';

   /* The user must pass five command-line arguments. */

   if (argc != 6)
      {
      printf ("Usage: api_xmt_pace drv_name vpi# vci# pkt_size kbit_rate\n");
      printf ("\n");
      printf ("   Where drv_name is either lpa or lta and its instance value\n");
      printf ("         vpi# is the vpi of interest\n");
      printf ("         vci# is the vci of interest\n");
      printf ("         pkt_size is packet size in bytes, 0<pkt_size<65536\n");
      printf ("         kbit_rate is the ((bits/sec) / 1000) desired\n");
      exit(1);
      }

   /* Make sure the user is attempting to communicate with a supported 
   *    driver service.
   */

   if (strncmp (argv[1], "lpa", 3) &&
       strncmp (argv[1], "lta", 3) != 0)
      {
      printf ("Unsupported driver service name\n");
      exit (-1);
      }

   /* Copy the driver service instance into the conn_id structure. */

   strncpy (&drv_name[0], argv[1], (strlen (argv[1]) + 1));
   for (i = 0; i < strlen (&drv_name[0]); i++)
      {
      if ((drv_name[i] >= '0') && (drv_name[i] <= '9'))
         {
         sscanf (&drv_name[i], "%d", &conn_id.device_number);
         break;
         }
      }

   /* Scan the user's command-line parameters and convert into the correct
   *    type.
   */

   sscanf (argv[2], "%d", &conn_id.vpi);
   sscanf (argv[3], "%d", &conn_id.vci);
   sscanf (argv[4], "%d", &pkt_size);
   sscanf (argv[5], "%d", &bit_rate);

   /* Set the local buffer to a hexadecimal incrementing pattern. */

   value = 1;
   for (i = 0; i < SIZE; i++)
      {
      buff[i] = value;
      value += 1;
      if (value > 0xF) value = 1;
      }

   /* Open an AAL, Datagram socket. */

   printf ("[ Creating ATM Socket ]\n");
   sock = socket (AF_AAL, SOCK_DGRAM, 0);   
   if (sock == -1)
      {
      printf ("[ Socket call failed, error: %d ]\n", errtext (errno));
      exit (1);
      }

   /* Get the driver's hard mtu size.  This value is required for establishing
   *    the fmtu and bmtu for the PVC.
   */

   device_info.device_number   = conn_id.device_number;
   device_info.driver_name     = drv_name;
   device_info.driver_name_len = 32;
   if (ioctl (sock, SIO_GET_DEVICE_INFO, &device_info) == -1)
      {
      printf ("Device %d inaccessible, socket error: %d\n",
              device_info.device_number,
              errno);
      }


   /* Set the params structure to its defaults.  Then, set the params structure
   *    to the known values for the CBR service.  Note the following calculation
   *    for the forward and backward peak cell rates (cells/sec).
   *
   *    C/S = BitRate / ((BitsPerByte) * (BytesPerCell))
   *    Note that (8 bits/byte * 53 bytes/cell) = 424 bits/cell
   */

   cell_rate            = (bit_rate * 1000) / 424;

   ATM_DEFAULTS (&params);
   params.fmtu          = device_info.hard_mtu;
   params.bmtu          = device_info.hard_mtu;
   params.fpeakcr[1]    = cell_rate;
   params.bpeakcr[1]    = cell_rate;
   params.flags         = ATM_FLAG_PACING;

   /* The CMM needs to know what to pass to the driver when an
   *    pvc association takes place.
   */

   if (ioctl (sock, SIO_SET_ATM_PARAMETERS, &params) == -1)
      {
      printf ("[ param ioctl failed, error: %d ]\n", errno);
      exit (1);
      }

   /* Notify the user of the association. */

   printf ("[ Associating Socket with PVC: %s/%d/%d ]\n",
	        &drv_name[0],
           conn_id.vpi,
           conn_id.vci);

   /* Create and open the PVC. */

   if (ioctl (sock, SIO_ASSOCIATE_PVC, &conn_id) == -1)
      {
      printf ("[ socket ioctl failed, error: %d ]\n", errno);
      exit (1);
      }

   /* Send the internal packet continuously.  This transmission ends when the
   *    user interrupts the executable with a <ctrl-c>.
   */

   while (TRUE)
      {
      i = send (sock, buff, pkt_size, 0);
      if (i == -1)
         {
         a--;
         if (errno != ENOBUFS) printf ("xmt fail - err = %d\n", errno);
         }
      else
         {
         printf (".");
         fflush (stdout);
         }
      }

   /* Close the socket and release all associated resources. */

   close (sock);
   return (1);
   }

